"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _child_process = require("child_process");
var _fs = require("fs");
var _logger = require("./logger");
var _http = _interopRequireDefault(require("http"));
var _path = _interopRequireDefault(require("path"));
var _querystring = _interopRequireDefault(require("querystring"));
var _FileUtilities = _interopRequireDefault(require("./FileUtilities"));
var _SonarQube = _interopRequireDefault(require("../core/sonar-dependencies/SonarQube"));
class Utility {
  /**
   * @function requestAPI - native request api 
   * @param {JSON} apiDetails 
   * @param {JSON} body 
   * @param {JSON} options 
   * @returns Promise<JSON>
   */

  static requestAPI(apiDetails, body, options = {}) {
    return new Promise(resolve => {
      var request = _http.default.request(apiDetails, function (res) {
        var chunks = [];
        res.on("data", function (chunk) {
          chunks.push(chunk);
        });
        res.on("end", function (chunk) {
          var body = Buffer.concat(chunks);
          resolve(body.toString());
        });
        res.on("error", function (error) {
          _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
        });
      });
      request.write(body);
      request.end();
    });
  }

  /**
   *@function requestUtilsCLI - method execute curl cli request
   * @returns {Promise<Buffer>}
   */

  static requestUtilsCLI(command) {
    return (0, _child_process.execSync)(command, {
      shell: true
    });
  }

  /**
   * @function createBranchDiff - make giff diff api via curl api
   * @returns {void}
   */

  static createBranchDiff() {
    var command;
    var {
      token,
      gitEndPoint,
      branchDiffPath,
      compareBranch,
      projectId
    } = _FileUtilities.default.getLintConfiguration();
    if (projectId == "project-id") {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that project id is properly configured`);
      projectId = this.getGitProjectId();
    }
    const endPoint = `${gitEndPoint}/api/v4/projects/${projectId.toString()}/repository/compare?from=${compareBranch}&to=${_SonarQube.default.getBranchName()}`;
    try {
      command = `curl --header "PRIVATE-TOKEN:${this.decrypt(token, 12)}" "${endPoint}"`;
      _FileUtilities.default.writeFileContents(branchDiffPath, this.requestUtilsCLI(command));
    } catch (err) {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, `\n INFO : If you are using a VPN and encounter an SSL certification issue, ensure that the proxy is enabled for SSH and shell connections.`);
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that you have access to this repository ${_SonarQube.default.getRepoName()}`);
    }
  }

  /**
   * @function getGitProjectId -  retrives project id of repository
   * @returns string
   */

  static getGitProjectId() {
    var command;
    const {
      token,
      gitEndPoint
    } = _FileUtilities.default.getLintConfiguration();
    const endPoint = `${gitEndPoint}/api/v4/projects?search=${_SonarQube.default.getRepoName()}`;
    try {
      command = (0, _child_process.execSync)(`curl --header "PRIVATE-TOKEN:${this.decrypt(token, 12)}" "${endPoint}"`);
      return JSON.parse(command.toString()).pop().id;
    } catch (err) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `make sure that you have access to this repository ${_SonarQube.default.getRepoName()}`);
      _logger.Logger.log(_logger.Logger.INFO_TYPE, err);
    }
  }

  /**
   * @function checkForExemptionNeed -  returns array of Paths after filteration of response of sonarQube issue api
   * @param {String} projectName  - project name of lint report
   * @returns Array<String>
   */

  static checkForExemptionNeed(projectName) {
    return new Promise(async resolve => {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, '\n Checking if an exemption has been raised for this branch');
      const {
        metricServerHost,
        meticHandler_api_token
      } = _FileUtilities.default.getLintConfiguration();
      try {
        const exclusionEndpoint = `${metricServerHost.toString()}/api/settings/values?keys=`;
        const exclusionParams = ['sonar.exclusions', 'sonar.issue.ignore.allfile', 'sonar.inclusions', 'sonar.issue.ignore.block', 'sonar.test.exclusions', 'sonar.issue.ignore.multicriteria', 'sonar.test.inclusions', 'sonar.issue.enforce.multicriteria', 'sonar.coverage.exclusions', `sonar.cpd.exclusions&component=${projectName}`];
        const api = exclusionEndpoint.concat(exclusionParams.join(',').endsWith(",") ? exclusionParams.substring(0, url.length() - 1) : exclusionParams);
        const command = (0, _child_process.execSync)(`curl --location "${api}" --header 'Authorization: Basic ${this.decrypt(meticHandler_api_token, 12)}'`).toString();
        (0, _fs.writeFileSync)(this.getExemptionFileInfoPath(), JSON.stringify(command.toString()));
      } catch (error) {
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Issue in applying exemption`);
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      }
      resolve(this.filterSonarQubeJSONResponse());
    });
  }

  /**
   * @function getExemptionFileInfoPath - returns path of exemption details
   * @returns string
   */

  static getExemptionFileInfoPath() {
    return _path.default.resolve(__dirname, "..", "..", "exemptionInfo.json");
  }

  /**
   * @function filterSonarQubeJSONResponse - convert json response to desired format
   * @returns Array<Object>
   */
  static async filterSonarQubeJSONResponse() {
    try {
      var _exemptionDetails$set;
      var criteriaKey = "sonar.issue.ignore.multicriteria";
      const exemptionDetails = JSON.parse(require(this.getExemptionFileInfoPath()));
      var excludedFiles;
      let isExemptionNeed = false;
      (exemptionDetails === null || exemptionDetails === void 0 || (_exemptionDetails$set = exemptionDetails.settings) === null || _exemptionDetails$set === void 0 ? void 0 : _exemptionDetails$set.length) && exemptionDetails.settings.forEach(element => {
        if (element.key == criteriaKey) {
          isExemptionNeed = true;
          excludedFiles = element.fieldValues.map(ele => ele.resourceKey);
        }
      });
      return {
        isExemptionApplied: isExemptionNeed,
        excludedFiles: [...new Set(excludedFiles)]
      };
    } catch (error) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Exemption not applied for this branch :(`);
      return {
        isExemptionApplied: false,
        excludedFiles: []
      };
    }
  }

  /**
   * @function deleteProject_SonarQube - this function delete project created in given sonarQube instance
   * @param {string} projectName - project name to delete  
   * @returns {Promiseany>}
   */
  static async deleteProject_SonarQube(projectName) {
    try {
      const {
        metricServerHost
      } = _FileUtilities.default.getLintConfiguration();
      const encryptedSonarToken = this.decrypt(meticHandler_api_token, 12);
      const {
        hostname
      } = new URL(metricServerHost);
      const requestBody = _querystring.default.stringify({
        project: projectName
      });
      const apiDetails = {
        method: "POST",
        hostname: hostname,
        path: "/api/projects/delete",
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
          Authorization: `Basic ${encryptedSonarToken}`
        },
        maxRedirects: 20
      };
      return await this.requestAPI(apiDetails, requestBody);
    } catch (error) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    }
  }

  /**
   * @function caesarCipher -  It shifts each letter by a specified number of places in the alphabet.
   * @param {string} str 
   * @param {string} shift 
   * @returns {string}
   */

  static caesarCipher(str, shift) {
    return str.split("").map(char => {
      let code = char.charCodeAt();
      if (code >= 65 && code <= 90) {
        return String.fromCharCode((code - 65 + shift) % 26 + 65);
      } else if (code >= 97 && code <= 122) {
        return String.fromCharCode((code - 97 + shift) % 26 + 97);
      }
      return char;
    }).join("");
  }
  /**
   * @function encrypt - Encrypts plaintext using the Caesar Cipher algorithm.
   * @param {string} plaintext 
   * @param {string} shift 
   * @returns {string}
   */

  static encrypt(plaintext, shift) {
    return this.caesarCipher(plaintext, shift);
  }

  /**
   * @function decrypt -  Decrypts ciphertext that was encrypted using the Caesar Cipher algorithm.
   * @param {string} ciphertext 
   * @param {string} shift 
   * @returns {string}
   */

  static decrypt(ciphertext, shift) {
    return this.caesarCipher(ciphertext, 26 - shift);
  }

  /**
   * @function encode - Encodes a given string into Base64 format.
   * @param {string} data 
   * @returns {string}
   */

  static encode(data) {
    return btoa(data);
  }

  /**
   * @function decode - Decodes a Base64-encoded string back to its original form.
   * @param {string} encode_data 
   * @returns {string}
   */

  static decode(encode_data) {
    return atob(encode_data);
  }
}
var _default = exports.default = Utility;